'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = babelPluginTransformMetadata;

var _path = require('path');

var _getTypesInfo = require('./visitors/getTypesInfo');

var _getTypesInfo2 = _interopRequireDefault(_getTypesInfo);

var _addReflections = require('./visitors/addReflections');

var _addReflections2 = _interopRequireDefault(_addReflections);

var _createTypeForAnnotation = require('./factories/createTypeForAnnotation');

var _createTypeForAnnotation2 = _interopRequireDefault(_createTypeForAnnotation);

var _createCreateCreateGenericTypeMetadata = require('./factories/createCreateCreateGenericTypeMetadata');

var _createCreateCreateGenericTypeMetadata2 = _interopRequireDefault(_createCreateCreateGenericTypeMetadata);

var _createCreateCreateObjectTypeMetadata = require('./factories/createCreateCreateObjectTypeMetadata');

var _createCreateCreateObjectTypeMetadata2 = _interopRequireDefault(_createCreateCreateObjectTypeMetadata);

var _createTypeForAnnotations = require('./factories/createTypeForAnnotations');

var _createTypeForAnnotations2 = _interopRequireDefault(_createTypeForAnnotations);

var _createGetUniqueTypeName = require('./factories/createGetUniqueTypeName');

var _createGetUniqueTypeName2 = _interopRequireDefault(_createGetUniqueTypeName);

var _createParentPathInsertAfter = require('./modifiers/createParentPathInsertAfter');

var _createParentPathInsertAfter2 = _interopRequireDefault(_createParentPathInsertAfter);

var _createReplaceMagicTypeCasts = require('./modifiers/createReplaceMagicTypeCasts');

var _createReplaceMagicTypeCasts2 = _interopRequireDefault(_createReplaceMagicTypeCasts);

var _createInjectParamTypes = require('./metaCreators/createInjectParamTypes');

var _createInjectParamTypes2 = _interopRequireDefault(_createInjectParamTypes);

var _createInsertFactory = require('./modifiers/createInsertFactory');

var _createInsertFactory2 = _interopRequireDefault(_createInsertFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaults = {
    typeNameStrategy: 'typeName',
    paramKey: 'design:paramtypes',
    typeKey: 'design:subtype',
    onlyExports: false,
    addDebugId: false,
    injectPrefix: '_rdi',
    ambiantTypeCastImport: 'babel-plugin-transform-metadata/_',
    ambiantDepsImport: 'babel-plugin-transform-metadata/Deps',
    jsxPragma: '_t'
};

function babelPluginTransformMetadata(_ref) {
    var t = _ref.types;

    var cnf = void 0;
    return {
        visitor: {
            Program: function Program(path, _ref2) {
                var opts = _ref2.opts,
                    file = _ref2.file;

                if (!cnf) {
                    cnf = _extends({}, defaults, opts);
                }
                var prefix = process ? process.cwd() : '';
                var filename = !cnf.addDebugId || file.opts.filename === 'unknown' ? null : (0, _path.basename)(prefix) + file.opts.filename.substring(prefix.length);
                var getUniqueTypeName = (0, _createGetUniqueTypeName2.default)(cnf.typeNameStrategy);
                var state = {
                    getUniqueTypeName: getUniqueTypeName,
                    ambiantTypeCastImport: cnf.ambiantTypeCastImport,
                    ambiantDepsImport: cnf.ambiantDepsImport,

                    ambiantTypeCast: null,
                    internalTypes: new Map(),
                    externalTypeNames: new Map(),
                    exportNames: new Map(),
                    functionsWithJsx: new Set()
                };
                path.traverse(_getTypesInfo2.default, state);
                var replaceMagicTypeCasts = (0, _createReplaceMagicTypeCasts2.default)(t, state.externalTypeNames);
                var createCreateObjectTypeMetadata = (0, _createCreateCreateObjectTypeMetadata2.default)(t);
                var createCreateGenericTypeMetadata = (0, _createCreateCreateGenericTypeMetadata2.default)(t, state.externalTypeNames, state.internalTypes);
                var typeForAnnotation = (0, _createTypeForAnnotation2.default)(t, state.externalTypeNames, createCreateObjectTypeMetadata, createCreateGenericTypeMetadata);
                var typeForAnnotations = (0, _createTypeForAnnotations2.default)(typeForAnnotation);

                var injectParamTypes = (0, _createInjectParamTypes2.default)(t, cnf.injectPrefix, typeForAnnotations, cnf.paramKey, cnf.typeKey, state.functionsWithJsx, filename);
                var parentPathInsertAfter = (0, _createParentPathInsertAfter2.default)(injectParamTypes);
                var reflectionState = {
                    t: t,
                    magicTypeCasts: [],
                    parentPaths: [],
                    onlyExports: cnf.onlyExports,
                    exportNames: state.exportNames,
                    magicTypeCastExpression: state.ambiantTypeCast ? state.ambiantTypeCast.node.specifiers[0].local.name : ''
                };

                path.traverse(_addReflections2.default, reflectionState);

                reflectionState.magicTypeCasts.forEach(replaceMagicTypeCasts);
                reflectionState.parentPaths.forEach(parentPathInsertAfter);
                if (cnf.jsxPragma) {
                    state.functionsWithJsx.forEach((0, _createInsertFactory2.default)(t, cnf.jsxPragma));
                }

                if (state.ambiantTypeCast) {
                    state.ambiantTypeCast.remove();
                }
            }
        }
    };
}
//# sourceMappingURL=index.js.map