'use strict';

exports.__esModule = true;
exports.default = createInjectParamTypes;
function createInjectParamTypes(t, injectPrefixRaw, typeForAnnotations, paramKey, typeKey, functionsWithJsx, filename) {
    var metaDataId = injectPrefixRaw ? null : t.memberExpression(t.identifier('Reflect'), t.identifier('defineMetadata'));
    var injectPrefix = injectPrefix || '_rdi';
    var argId = t.identifier(injectPrefix + 'Arg');
    var jsxId = t.identifier(injectPrefix + 'Jsx');
    var fnId = t.identifier(injectPrefix + 'Fn');
    var debugId = t.identifier(injectPrefix + 'Dbg');

    // console.log(functionsWithJsx)
    return function injectParamTypes(rawTypes, target, node) {
        var body = [];
        var type = node.type;
        var isJsx = functionsWithJsx.has(node);

        if (rawTypes.length) {
            var types = void 0;
            if (isJsx) {
                types = rawTypes.length > 1 ? [rawTypes[1]] : [];
            } else {
                types = rawTypes;
            }
            var typeArgs = typeForAnnotations(types, node.typeParameters ? node.typeParameters.params : null);
            if (typeArgs.length) {
                if (metaDataId) {
                    body.push(t.expressionStatement(t.callExpression(metaDataId, [t.stringLiteral(paramKey), t.arrayExpression(typeArgs), target])));
                } else {
                    body.push(t.expressionStatement(t.assignmentExpression('=', t.memberExpression(target, argId), t.arrayExpression(typeArgs))));
                }
            }

            for (var i = 0; i < types.length; i++) {
                var argType = types[i];
                var decorators = argType.decorators || [];
                for (var j = 0; j < decorators.length; j++) {
                    body.push(t.expressionStatement(t.callExpression(decorators[j].expression, [target, t.identifier('null'), t.identifier(i.toString())])));
                }
            }
        }

        if (type === 'FunctionDeclaration' || type === 'FunctionExpression' || type === 'ArrowFunctionExpression') {
            if (metaDataId) {
                body.push(t.expressionStatement(t.callExpression(metaDataId, [t.stringLiteral(typeKey), t.stringLiteral(isJsx ? 'jsx' : 'func'), target])));
            } else {
                body.push(t.expressionStatement(t.assignmentExpression('=', t.memberExpression(target, isJsx ? jsxId : fnId), t.booleanLiteral(true))));
            }
            if (filename) {
                body.push(t.expressionStatement(t.assignmentExpression('=', t.memberExpression(target, debugId), t.stringLiteral(filename + '#' + target.name))));
            }
        }

        return body;
    };
}
//# sourceMappingURL=createInjectParamTypes.js.map